#!/usr/bin/python
import numpy as np
from sympy.physics.quantum.dagger import Dagger
import cmath
from cmath import*
import math
from lmfit import Model
import time
import matplotlib.pyplot as plt


a = (2.4601e-10)*5
agrp = a/5.
hbar = 6.582119e-16	#hbar in units eVs
pi = math.pi		#constant pi = 3.1415...
Ry = 13.605693009	#Rydberg constant
lambdaR = -0.00091528
lambdaIA = 0.00080827
lambdaIB = 0.00124670
lambdaPIAA = 0
lambdaPIAB = 0
delta = 0.00229514
vf = 7.8768e+05
BA = 0.0
BB = 0.0
EK = 0.01147414
XI = 0.0
#DFT daten#

tau = 1	#choose the valley

kx = []
ky = []

if tau > 0:
	data = np.loadtxt('./bandsK/DFTbandsK.dat')
	ED = EK
	k1 = data[:,0]-0.3333333
	k2 = data[:,1]-0.3333333
	kmax=len(k1)
	for i in range(0,kmax):
		kx.append(-((4*pi/(sqrt(3)*a))*(k1[i]*sqrt(3.)/2.)).real)
		ky.append(-((4*pi/(sqrt(3)*a))*(k1[i]*(1./2.)+k2[i])).real)

if tau < 0:
	data = np.loadtxt('./bandsKp/DFTbandsKp.dat')
	ED = EK
	k1 = data[:,0]+0.3333333
	k2 = data[:,1]+0.3333333
	kmax=len(k1)
	for i in range(0,kmax):
		kx.append(-((4*pi/(sqrt(3)*a))*(k1[i]*sqrt(3.)/2.)).real)
		ky.append(-((4*pi/(sqrt(3)*a))*(k1[i]*(1./2.)+k2[i])).real)



#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def Hamiltonian(kx,ky,tau,delta,lambdaIB,lambdaIA,lambdaR,ED,vf,lambdaPIAA,lambdaPIAB,BA,BB,XI):

	H = np.array([[-BA+delta+tau*lambdaIA+tau*XI, 	-lambdaPIAA*complex(ky,kx)*agrp, 	hbar*vf*complex(tau*kx,ky),		complex(0.0,lambdaR+tau*lambdaR)],
		      [-lambdaPIAA*complex(ky,-kx)*agrp, 	BA+delta-tau*lambdaIA+tau*XI, 	complex(0.0,lambdaR-tau*lambdaR), 	hbar*vf*complex(tau*kx,ky)],
		      [hbar*vf*complex(tau*kx,-ky), 		complex(0.0,-lambdaR+tau*lambdaR), 	-BB-delta-tau*lambdaIB+tau*XI, 		-lambdaPIAB*complex(ky,kx)*agrp],
		      [complex(0.0,-lambdaR-tau*lambdaR), 	hbar*vf*complex(tau*kx,-ky), 	-lambdaPIAB*complex(ky,-kx)*agrp,	BB-delta+tau*lambdaIB+tau*XI]])

	Sx = np.array([ [0,1,0,0],
			[1,0,0,0],
			[0,0,0,1],
			[0,0,1,0]])

	Sy = np.array([ [0,complex(0,-1),0,0],
			[complex(0,1),0,0,0],
			[0,0,0,complex(0,-1)],
			[0,0,complex(0,1),0]])

	Sz = np.array([ [1,0,0,0],
			[0,-1,0,0],
			[0,0,1,0],
			[0,0,0,-1]])

	evals, evecs = np.linalg.eigh(H)

	ew=[[],[],[],[]]
	spinx=[[],[],[],[]]
	spiny=[[],[],[],[]]
	spinz=[[],[],[],[]]
	#verify=[[],[],[],[]]
	
	for i in range(0,4):
		ew[i] = evals[i]+ED
		EV = evecs[:,i]
		DEV = Dagger(evecs[:,i])
		spinx[i]=0.5*(DEV.dot(Sx.dot(EV))).real
		spiny[i]=0.5*(DEV.dot(Sy.dot(EV))).real
		spinz[i]=0.5*(DEV.dot(Sz.dot(EV))).real	
		#verify[i] = H.dot(evecs[:,i]) - evals[i] * evecs[:,i] #used to verify if H*EV-EW*EV = 0 for every EV,EW pair
	
	return ew[0],ew[1],ew[2],ew[3],spinx[0],spinx[1],spinx[2],spinx[3],spiny[0],spiny[1],spiny[2],spiny[3],spinz[0],spinz[1],spinz[2],spinz[3]

	
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
print "#|k|,E1M,E2M,E3M,E4M,SX1M,SX2M,SX3M,SX4M,SY1M,SY2M,SY3M,SY4M,SZ1M,SZ2M,SZ3M,SZ4M,E1D,E2D,E3D,E4D,SX1D,SX2D,SX3D,SX4D,SY1D,SY2D,SY3D,SY4D,SZ1D,SZ2D,SZ3D,SZ4D"
for i in range(0,kmax):
	EDFT = data[i][2:6]
	SXDFT = data[i][6:10]
	SYDFT = data[i][10:14]
	SZDFT = data[i][14:18]
	EW = Hamiltonian(kx[i],ky[i],tau,delta,lambdaIB,lambdaIA,lambdaR,ED,vf,lambdaPIAA,lambdaPIAB,BA,BB,XI)
	if kx[i] < 0:
		print -(sqrt(kx[i]**2+ky[i]**2)*1e-10).real,EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EDFT[0],EDFT[1],EDFT[2],EDFT[3],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3]
	if kx[i] >= 0:
		print (sqrt(kx[i]**2+ky[i]**2)*1e-10).real,EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EDFT[0],EDFT[1],EDFT[2],EDFT[3],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3]


print ""
print "#",delta,vf,lambdaIA,lambdaIB,lambdaR,lambdaPIAA,lambdaPIAB,ED


