#!/usr/bin/python
import numpy as np
from sympy.physics.quantum.dagger import Dagger
import cmath
from cmath import*
import math
from lmfit import Model
import time
import matplotlib.pyplot as plt
from numpy import conjugate

a = (2.46e-10)
hbar = 6.582119e-16
pi = math.pi
tau = 1
Ry = 13.605693009
vf = 8.0e+05	#Fermi velocity
ED = -1.38874774	#Doping level, as DFT sets the energies with respect to Ef, but the model around Ef = 0
delta = 0.01707507	#gap parameter, staggered potential
lambdaIA = -9.3803e-06	#intrinsic SOC sublattice A
lambdaIB = -4.9902e-06	#intrinsic SOC sublattice B
lambdaR = 1.0647e-05	#Rashba SOC
lambdaPIAA = 3.7710e-05 #PIA SOC on sublattice A
lambdaPIAB =3.3460e-05 #PIA SOC on sublattice B
BA = 0.0				#Exchange coupling on sublattice A
BB = 0.0				#Exchange coupling on sublattice B
hopping = hbar*vf*2.0/(sqrt(3)*a)	#hopping 

kx = []
ky = []
qx = []
qy = []

#DFT daten#

data = np.loadtxt('bandsMKG_DFT.dat')

k1 = data[:,0]
k2 = data[:,1]

kmax=len(k1)

for i in range(0,kmax):
	kx.append(-((4*pi/(sqrt(3)*a))*(k1[i]+(1./2.)*k2[i])).real)
	ky.append(-((4*pi/(sqrt(3)*a))*(k2[i]*sqrt(3.)/2.)).real)

for i in range(0,kmax):
		qx.append(4*pi/(3*a)+kx[i])
		qy.append(0.0+ky[i])


def Hamiltonian(qx,qy,tau,delta,lambdaIB,lambdaIA,lambdaR,ED,vf,lambdaPIAA,lambdaPIAB):
	
	Forb = 1+2*cos(qx*a/2.0)*exp(complex(0.0,qy*a*sqrt(3)/2.0))
	Fintrinsic = -(2.0/(3*sqrt(3)))*(sin(qx*a)+sin(-qx*a/2.0+qy*a*sqrt(3)/2.0)+sin(-qx*a/2.0-qy*a*sqrt(3)/2.0))
	Fpia = complex(0.0,(4.0/3.0)*(sin(qx*a)+exp(complex(0.0,-2*pi/3.0))*sin(-qx*a/2.0+qy*a*sqrt(3)/2.0)+exp(complex(0.0,2*pi/3.0))*sin(-qx*a/2.0-qy*a*sqrt(3)/2.0)))
	Frashba = (2.0/3.0)*(1+exp(complex(0,-2*pi/3.0))*exp(complex(0.0,a*(qx+sqrt(3)*qy)/2.0))+exp(complex(0,2*pi/3.0))*exp(complex(0.0,a*(-qx+sqrt(3)*qy)/2.0)))
	Frashba2 = (2.0/3.0)*(1+exp(complex(0,-2*pi/3.0))*exp(complex(0.0,a*(-qx-sqrt(3)*qy)/2.0))+exp(complex(0,2*pi/3.0))*exp(complex(0.0,a*(qx-sqrt(3)*qy)/2.0)))

	H = np.array([[delta+lambdaIA*Fintrinsic, 	lambdaPIAA*Fpia, 		-hopping*Forb,					complex(0.0,lambdaR*Frashba)],
		      [lambdaPIAA*conjugate(Fpia), 	delta-lambdaIA*Fintrinsic, 	complex(0.0,lambdaR*conjugate(Frashba2)), 	-hopping*Forb],
		      [-hopping*conjugate(Forb), 	complex(0.0,-lambdaR*Frashba2), 	-delta-lambdaIB*Fintrinsic, 		lambdaPIAB*Fpia],
		      [complex(0.0,-lambdaR*conjugate(Frashba)), -hopping*conjugate(Forb), 	lambdaPIAB*conjugate(Fpia),	-delta+lambdaIB*Fintrinsic]])

	Sx = np.array([ [0,1,0,0],
			[1,0,0,0],
			[0,0,0,1],
			[0,0,1,0]])

	Sy = np.array([ [0,complex(0,-1),0,0],
			[complex(0,1),0,0,0],
			[0,0,0,complex(0,-1)],
			[0,0,complex(0,1),0]])

	Sz = np.array([ [1,0,0,0],
			[0,-1,0,0],
			[0,0,1,0],
			[0,0,0,-1]])

	evals, evecs = np.linalg.eigh(H)

	ew=[[],[],[],[]]
	spinx=[[],[],[],[]]
	spiny=[[],[],[],[]]
	spinz=[[],[],[],[]]
	#verify=[[],[],[],[]]
	
	for i in range(0,4):
		ew[i] = evals[i]#+ED
		EV = evecs[:,i]
		DEV = Dagger(evecs[:,i])
		spinx[i]=(DEV.dot(Sx.dot(EV))).real
		spiny[i]=(DEV.dot(Sy.dot(EV))).real
		spinz[i]=(DEV.dot(Sz.dot(EV))).real	
		#verify[i] = H.dot(evecs[:,i]) - evals[i] * evecs[:,i] #used to verify if H*EV-EW*EV = 0 for every EV,EW pair
	
	return ew[0],ew[1],ew[2],ew[3],spinx[0],spinx[1],spinx[2],spinx[3],spiny[0],spiny[1],spiny[2],spiny[3],spinz[0],spinz[1],spinz[2],spinz[3]

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


print "#|k|, E1Mod, E2Mod, E3Mod, E4Mod, E1DFT, E2DFT, E3DFT, E4DFT, SZ1Mod, SZ2Mod, SZ3Mod, SZ4Mod, SZ1DFT, SZ2DFT, SZ3DFT, SZ4DFT"
for i in range(len(qx)):
	EDFT = data[i][2:6]*Ry-ED
	SXDFT = data[i][6:10]
	SYDFT = data[i][10:14]
	SZDFT = data[i][14:18]
	EW = Hamiltonian(qx[i],qy[i],tau,delta,lambdaIB,lambdaIA,lambdaR,ED,vf,lambdaPIAA,lambdaPIAB)
	if k2[i]==k2[i-1]:
		print k1[i],k2[i], EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EDFT[0],EDFT[1],EDFT[2],EDFT[3],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3]
	else:
		print " "
		print k1[i],k2[i], EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EDFT[0],EDFT[1],EDFT[2],EDFT[3],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3]







